
import rospy
from prompt_toolkit import PromptSession
from pygments.lexers.scripting import LuaLexer
from prompt_toolkit.lexers import PygmentsLexer
from std_msgs.msg import String
from schpin_msgs.srv import LuaDef, LuaCall
from lupa import LuaRuntime
import json

def on_feedback(msg):
    print(msg.data + "\n")

def callback_handler(name, x):
    print("calling: ", name, " with args: ", x)
    resp = call_serv(name, json.dumps(x))
    if resp.success:
        return json.loads(resp.response)
    print("Error: ", resp.error)

rospy.init_node("lua_console")

call_serv = rospy.ServiceProxy("/control/lua/call", LuaCall)
def_serv = rospy.ServiceProxy('/control/lua/def', LuaDef)

lua_def_string = def_serv().definition
lua_def = json.loads(lua_def_string)

lua = LuaRuntime()
reg = lua.eval('function(keyword, val) _G[keyword] = val end')

for item in lua_def:
    if item["type"] == "callback":
        print("registering ", item["name"])
        reg(item["name"], lambda x: callback_handler(item["name"], x))
    elif item["type"] == "global variable":
        continue
    elif item["type"] == "called lua procedure":
        continue
    elif item["type"] == "lua procedure":
        print("registering ", item["name"])
        reg(item["name"], lambda *x: callback_handler(item["name"], x))

session = PromptSession()

while not rospy.is_shutdown():
    cmd = session.prompt(">", lexer=PygmentsLexer(LuaLexer))
    lua.execute(cmd)
