#include "node/components/control.h"

using namespace schpin;

control_component::control_component( const ros::NodeHandle& handle, const node_context& context )
  : handle_( handle )
  , state_( boot_state_machine{ map_f_to_v( context.sgroup.get_servos(),
                                            []( const servo& s ) {  //
                                                    return s.get_id();
                                            } ) } )
  , ctl_serv_( handle_.advertiseService( "/control/ctl", &control_component::on_ctl, this ) )
{
}

bool control_component::on_ctl( schpin_msgs::BlobCtl::Request& req, schpin_msgs::BlobCtl::Response& res )
{
        if ( !std::holds_alternative< idle_state_machine >( state_ ) ) {
                res.res = schpin_msgs::BlobCtl::Response::BUSY;
                return true;
        }

        state_.emplace< lua_vm_machine >( req.data );

        return true;
}

void control_component::tick( const node_context& context, sm_feedback_f fb_fun )
{
        std::visit(
            [&]( auto& state ) {
                    state.tick( context,
                                fb_fun )
                        .optionally( [&]( auto error ) {  //
                                SCHPIN_ERROR_LOG( "control", simplified_error{ error } );
                        } );
            },
            state_ );

        bool finished = std::visit(
            [&]( auto& state ) {  //
                    return state.finished();
            },
            state_ );

        if ( !finished ) {
                return;
        }

        state_ << match(
            []( const idle_state_machine& ) {  //
            },
            [&]( const statelog_state_machine& ) {  //
                    state_.emplace< idle_state_machine >();
            },
            [&]( const boot_state_machine& ) {  //
                    state_.emplace< statelog_state_machine >();
            },
            [&]( const lua_vm_machine& ) {  //
                    state_.emplace< idle_state_machine >();
            } );
}

void control_component::on_feedback( const fw_node_group_variant& v,
                  const node_context&              context,
                  sm_feedback_f                    fb_fun )
{
        std::visit(
            [&]( auto& state ) {
                    state.on_cmd( v, context, fb_fun ).optionally( [&]( auto e ) {
                            SCHPIN_ERROR_LOG( "control", simplified_error{ e } );
                    } );
            },
            state_ );
}
