#include "shared/protocol.h"

#include <deque>
#include <ros/ros.h>
#include <schpin_lib/logging.h>

#pragma once

namespace schpin
{
class msg_history
{
        struct msg_record
        {
                node_fw_message msg;
                ctl_id              id;
                ros::Time           sent_at;
        };
        std::deque< msg_record > history_;

      public:
        using const_iterator = typename std::deque< msg_record >::const_iterator;

        void insert( ctl_id cid, node_fw_message m )
        {
                history_.push_back( msg_record{ m, cid, ros::Time::now() } );
        }

        const_iterator begin() const
        {
                return history_.begin();
        }

        const_iterator find( ctl_id id ) const
        {
                return em::find_if( history_,
                                    [&]( const msg_record& rec ) { return rec.id == id; } );
        }

        void try_log( ctl_id id ) const
        {
                const_iterator iter = find( id );
                if ( iter == end() ) {
                        SCHPIN_INFO_LOG( "msg_history",
                                         "History record for msg with id " << id << " is missing" );
                } else {
                        SCHPIN_INFO_LOG( "msg_history",
                                         "Msg with id " << id << " was sent at " << iter->sent_at
                                                        << " and is:" << iter->msg );
                }
        }

        const_iterator end() const
        {
                return history_.end();
        }

        void drop( ctl_id cid )
        {
                auto iter = find( cid );
                if ( iter == end() ) {
                        return;
                }
                history_.erase( iter );
        }
};

}  // namespace schpin
