#include "node/leg/model.h"

#include <vector>

#pragma once

namespace schpin
{

class robot_model
{
        std::vector< leg_model > legs_;

      public:
        robot_model( robot_model&& ) = default;
        robot_model& operator=( robot_model&& ) = default;

        robot_model( const robot_model& ) = delete;
        robot_model& operator=( const robot_model& ) = delete;

        robot_model( std::vector< leg_model > legs )
          : legs_( std::move( legs ) )
        {
        }

        const std::vector< leg_model >& get_legs() const
        {
                return legs_;
        }
};

em::either< robot_model, load_error > load_robot_model( const urdf::Model&      model,
                                                        const srdf_information& srdf_info );

inline std::ostream& operator<<( std::ostream& os, const robot_model& r )
{
        os << "robot_model: " << std::endl;
        for ( const leg_model& l : r.get_legs() ) {
                os << l << std::endl;
        }
        return os;
}

}  // namespace schpin
