#include "node/leg/state.h"
#include "node/robot/model.h"

#pragma once

namespace schpin
{

struct robot_state
{
        std::vector< leg_state > legs;
        pose< world_frame >      body_pose;

        robot_state( const robot_model& m )
        {
                for ( const leg_model& l : m.get_legs() ) {
                        legs.push_back( leg_state{ l } );
                }
        }

        bool update_joint_state( const robot_model& m, const std::string& jname, em::angle a )
        {
                return em::any_of(
                    em::zip( legs, m.get_legs() ),
                    em::uncurry | [&]( leg_state& state, auto& leg ) {
                            return state.update_joint_state( leg, jname, a );
                    } );
        }
};

inline std::ostream& operator<<( std::ostream& os, const robot_state& state )
{
        os << "body pose: " << state.body_pose << ", legs: \n";
        for ( const leg_state& lstate : state.legs ) {
                os << lstate << "\n";
        }
        return os;
}

}  // namespace schpin
