#include "emlabcpp/either.h"

#include <ostream>
#include <vector>

#pragma once

namespace schpin
{

namespace em = emlabcpp;

// Standard stream operator of em::either directly prints em::either left or right item.
template < typename LH, typename RH >
inline std::ostream& operator<<( std::ostream& os, const em::either< LH, RH >& env )
{
        env.match(
            [&]( const LH& lh ) {  //
                    os << lh;
            },
            [&]( const RH& rh ) {  //
                    os << rh;
            } );
        return os;
}

// Structure containing content of em::eithers stored in left/right vectors
template < typename LH, typename RH >
struct partitioned_eithers
{
        std::vector< LH > left;
        std::vector< RH > right;
};

// Partitions a collection of em::eithers into vector of left values and vector of right values,
// this implies that all em::eithers have same type.
template < typename LH, typename RH >
partitioned_eithers< LH, RH > partition_eithers( std::vector< em::either< LH, RH > > data )
{
        partitioned_eithers< LH, RH > res;
        for ( em::either< LH, RH >& item : data ) {
                std::move( item ).match(
                    [&]( LH&& lh_item ) { res.left.push_back( std::move( lh_item ) ); },
                    [&]( RH&& rh_item ) { res.right.push_back( std::move( rh_item ) ); } );
        }
        return res;
}
}  // namespace schpin
