#include "schpin_lib/geom/quaternion.h"

#include <geometry_msgs/Quaternion.h>
#include <nlohmann/json.hpp>

#pragma once

namespace schpin
{

template < typename Tag >
constexpr quaternion< Tag > msg_to_quaternion( const geometry_msgs::Quaternion& msg )
{
        return quaternion< Tag >{ float( msg.x ), float( msg.y ), float( msg.z ), float( msg.w ) };
}

template < typename Tag >
inline geometry_msgs::Quaternion quaternion_to_msg( const quaternion< Tag >& q )
{
        geometry_msgs::Quaternion msg;
        msg.x = static_cast< double >( q[0] );
        msg.y = static_cast< double >( q[1] );
        msg.z = static_cast< double >( q[2] );
        msg.w = static_cast< double >( q[3] );
        return msg;
}

template < typename Tag >
inline void to_json( nlohmann::json& j, const quaternion< Tag >& quat )
{
        j = { quat[0], quat[1], quat[2], quat[3] };
}

template < typename Tag >
inline void from_json( const nlohmann::json& j, quaternion< Tag >& quat )
{
        quat = quaternion< Tag >{
            j.at( 0 ).get< float >(),
            j.at( 1 ).get< float >(),
            j.at( 2 ).get< float >(),
            j.at( 3 ).get< float >()  //
        };
}
template < typename Tag >
inline std::ostream& operator<<( std::ostream& os, const quaternion< Tag >& q )
{
        os << std::fixed << std::setprecision( 6 );
        return os << "[" << q[0] << "," << q[1] << "," << q[2] << "," << q[3] << "]";
}

}  // namespace schpin

template < typename Tag >
struct std::hash< schpin::quaternion< Tag > >
{
        std::size_t operator()( const schpin::quaternion< Tag >& quat ) const
        {
                std::stringstream ss;
                ss << quat;
                return std::hash< std::string >()( ss.str() );
        }
};
