#include "emlabcpp/iterators/access.h"
#include "emlabcpp/view.h"
#include "schpin_lib/types.h"

#ifdef SCHPIN_USE_NLOHMANN_JSON
#include <nlohmann/json.hpp>
#endif

#pragma once

namespace emlabcpp
{

// Output operator for the view, uses comma to separate the items in the view.
template < typename Iterator >
inline std::ostream& operator<<( std::ostream& os, const view< Iterator >& output )
{
        using value_type = decltype( *std::declval< Iterator >() );
        char delim       = ' ';
        for ( const value_type& item : output ) {
                os << delim << item;
                delim = ',';
        }
        return os;
}

}  // namespace emlabcpp

namespace em = emlabcpp;

namespace schpin
{

#ifdef SCHPIN_USE_NLOHMANN_JSON
// Serialization for the existing view of data using nlohmann's json library
template < typename Iter >
inline void to_json( nlohmann::json& j, const em::view< Iter >& v )
{
        for ( const auto& item : v ) {
                j.push_back( item );
        }
}
#endif

}  // namespace schpin
