#include <gtest/gtest.h>
#include <schpin_lib/lua.h>
#include <schpin_lib_exp/overload.h>

using namespace schpin;

struct empty_event
{
};

struct one_arg_event
{
        std::string arg;
};

struct multiarg_event
{
        std::string arg;
        float       val;
};

struct return_event
{
        std::string arg;
};

struct return_response
{
        std::string var;
        float       fvar;
};

struct optarg_event
{
        std::optional< std::string > opt_str;
};

struct array_event
{
        std::array< float, 10 > data;
};

struct slua_test_vm_def
{
        static auto get_def()
        {
                return std::make_tuple(
                    print_proc,
                    slua_proc(  //
                        "empty",
                        "Command without argument",
                        slua_struct< empty_event >() ),
                    slua_proc(  //
                        "onecmd",
                        "Command with one argument",
                        slua_struct< one_arg_event >(
                            lua_key_arg< std::string >{ "one", "One arg to this cmd" } ) ),
                    slua_proc(  //
                        "multicmd",
                        "Command with multiple arguments",
                        slua_struct< multiarg_event >(
                            lua_key_arg< std::string >{ "first",
                                                        "First argument that is a "
                                                        "string!" },
                            lua_key_arg< float >{ "second", "Second argument!" } ) ),
                    slua_func(  //
                        "returncmd",
                        "Command that returns value",
                        slua_struct< return_event >(
                            lua_key_arg< std::string >{ "arg", "Single argument!" } ),
                        slua_result< std::tuple< std::string, std::string > >{ "Result of "
                                                                               "function "
                                                                               "is result!" } ),
                    slua_proc(  //
                        "arrayproc",
                        "Procedure that has struct with an array as argument ",
                        slua_struct< array_event >(
                            lua_key_arg< std::array< float, 10 > >{ "data", "Array of data" } ) ),
                    slua_proc(  //
                        "opt_arg",
                        "Lua call with opt arg",
                        slua_struct< optarg_event >(  //
                            lua_opt_key_arg< std::string >{ "opt_str",
                                                            "optional string as "
                                                            "argument" } ) ) );
        }
};

TEST( lua, basic )
{
        using vm_type = slua_vm< slua_test_vm_def >;
        vm_type vm;

        std::string code = "print(\"wolololo\", 42);";
        vm.load( code ).optionally( [&]( slua_error e ) { FAIL() << e; } );

        auto event_matcher =
            match( [&]( lua_State*, slua_print_event e ) { EXPECT_EQ( e.msg, "wolololo\t42" ); },
                   [&]( lua_State*, empty_event ) { FAIL(); },
                   [&]( lua_State*, optarg_event ) { FAIL(); },
                   [&]( lua_State*, array_event ) { FAIL(); },
                   [&]( lua_State*, one_arg_event ) { FAIL(); },
                   [&]( lua_State*, multiarg_event ) { FAIL(); },
                   [&]( lua_State*, return_event ) -> std::tuple< std::string, std::string > {
                           return std::tuple< std::string, std::string >{ "1234", "5678" };
                   } );

        std::variant< lua_res, slua_error > res = vm.exec( event_matcher );

        res << match(
            [&]( lua_res ) {  //
            },
            [&]( slua_error e ) {  //
                    FAIL() << e;
            } );
}
