#include <functional>
#include <ostream>

#pragma once

namespace schpin
{

template < typename Graph >
struct graph_search_task
{
        using vertex_id_type = typename Graph::vertex_id_type;
        using price_type     = typename Graph::price_type;
        vertex_id_type current_id;
        vertex_id_type goal_id;
};

enum class graph_search_state
{
        FOUND_PATH,
        NOT_FINISHED,
        PATH_MISSING
};

inline std::ostream& operator<<( std::ostream& os, graph_search_state state )
{
        switch ( state ) {
                case graph_search_state::FOUND_PATH:
                        return os << "found path";
                case graph_search_state::NOT_FINISHED:
                        return os << "not finished";
                case graph_search_state::PATH_MISSING:
                        return os << "path missng";
        }
        return os;
}

}  // namespace schpin

template < typename Graph >
struct std::hash< schpin::graph_search_task< Graph > >
{
        std::size_t operator()( const schpin::graph_search_task< Graph >& g )
        {
                using vertex_id = typename Graph::VertexID;
                return std::hash< vertex_id >()( g.current_id ) ^
                       std::hash< vertex_id >()( g.goal_id );
        }
};
