#include "robot.h"
#include "robot/storage.h"

using namespace schpin;

int main( int argc, char* argv[] )
{

        ros::init( argc, argv, "planning_tester" );
        ros::NodeHandle handle{ "~" };

        std::string output_filename;
        if ( !handle.getParam( "output_file", output_filename ) ) {
                SCHPIN_WARN_LOG( "export_robot", "No output file specified" );
                return -1;
        }

        std::filesystem::path cache_prefix{ "~/.schpin/cache/" };

        urdf::Model model;
        model.initParam( "/robot_description" );

        std::string srdf;
        handle.getParam( "/robot_semantics", srdf );

        load_srdf_info( srdf )
            .convert_right( [&]( const xml_storage_error& e ) {
                    return E( robot_error ).attach( e ) << "Failed to load srdf for robot";
            } )
            .bind_left( [&]( const srdf_information& srdf_info ) {
                    return load_robot( cache_prefix, model, srdf_info );
            } )
            .match(
                [&]( robot&& robot ) {
                        std::ofstream out_stream{ output_filename };
                        SCHPIN_INFO_LOG( "export_robot", "Writing to " << output_filename );
                        if ( !out_stream ) {
                                SCHPIN_WARN_LOG( "export_robot",
                                                 "Failed to open file: " << strerror( errno ) );
                                return;
                        }
                        safe_store_json( out_stream, robot )
                            .optionally( [&]( const json_storage_error& e ) {
                                    SCHPIN_WARN_LOG( "export_robot", e );
                            } );
                },
                []( const robot_error& e ) { SCHPIN_WARN_LOG( "export_robot", e ); } );

        ros::spinOnce();

        return 0;
}
